#!/usr/bin/env python3

##### copy over from per_country.py
##### dont forget to copy DISPLAY_NAME over as well
##### ^^ end of countries and DISPLAY_NAME part
countries = [
    "USA",  # states in us_countries
    "Canada",  # provinces set in canada_countries
    "EU",  # eu countries in eu_countries
    # Europe
    "UK", "Serbia", "Albania", "Russia", "Ukraine", "Belarus", "Switzerland", "Moldova",
    "Bosnia and Herzegovina", "Iceland", "Faroe Islands",
    # APAC
    "Guam (US)", "Singapore", "Taiwan", "New Zealand", "Japan", "South Korea", "Cambodia",
    "Vietnam", "Malaysia", "Thailand", "Laos", "Philippines", "Indonesia", "Papua New Guinea", "Australia",
    # Africa
    "Seychelles", "Egypt", "Senegal", "Burkina Faso", "Nigeria", "Morocco",
    "Tunisia", "Mali", "Kenya", "South Africa", "Isle of Man",
    # Latin America
    "Peru", "Mexico", "Cuba", "Guatemala", "Costa Rica", "Brazil", "Bolivia", "Chile",
    "Argentina", "Uruguay", "Paraguay", "Colombia",
    # Middle East and Central Asia
    "Kazakhstan", "Uzbekistan", "Kyrgyzstan", "Mongolia", "Iraq", "Iran", "Saudi Arabia", "Jordan", "Israel", "Turkey",
    # Others
    "India", "Pakistan", "Nepal", "Falkland Islands", "Greenland",
    # regions
]

eu_countries = [
    "Czechia", "Netherlands", "Italy", "Slovakia", "Ireland", "Denmark", "Spain", "Luxembourg", "Slovenia",
    "Greece", "Sweden", "Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Estonia", "Finland", "France",
    "Germany", "Hungary", "Latvia", "Lithuania", "Malta", "Poland", "Portugal", "Romania",
]
countries.extend(eu_countries)

us_countries = ["Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado", "Connecticut", "Delaware",
    "Florida", "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana", "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
    "Maryland", "Massachusetts", "Michigan", "Minnesota", "Mississippi", "Missouri", "Montana", "Nebraska", "Nevada", "New Hampshire",
    "New Jersey", "New Mexico", "New York", "North Carolina", "North Dakota", "Ohio", "Oklahoma", "Oregon", "Pennsylvania",
    "Rhode Island", "South Carolina", "South Dakota", "Tennessee", "Texas", "Utah", "Vermont", "Virginia", "Washington", "West Virginia",
    "Wisconsin", "Wyoming"]
countries.extend(us_countries)

canada_countries = ["Alberta", "British Columbia", "Manitoba", "New Brunswick", "Newfoundland and Labrador", "Northwest Territories",
    "Nova Scotia", "Ontario", "Prince Edward Island", "Quebec", "Saskatchewan", "Yukon"]
countries.extend(canada_countries)

italy_regions = [
"Abruzzo (Italy region)", "Basilicata (Italy region)", 
"P.A. Bolzano (Italy region)",  # south tyrol
"Calabria (Italy region)", "Campania (Italy region)", "Emilia-Romagna (Italy region)", "Friuli Venezia Giulia (Italy region)",
"Lazio (Italy region)", "Liguria (Italy region)", "Lombardia (Italy region)", "Marche (Italy region)", "Molise (Italy region)",
"Piemonte (Italy region)", "Puglia (Italy region)", "Sardegna (Italy region)", "Sicilia (Italy region)", "Toscana (Italy region)",
"P.A. Trento (Italy region)", "Umbria (Italy region)", "Valle d'Aosta (Italy region)", "Veneto (Italy region)",
]
countries.extend(italy_regions)

DISPLAY_NAME = {
    "Czechia": "Czech Republic",
    "Guam (US)": "Guam",
}
for c in us_countries:
    if c == "New York":
        DISPLAY_NAME[c] = "New York (state) (US)"
    else:
        DISPLAY_NAME[c] = c + " (US)"
for c in canada_countries:
    DISPLAY_NAME[c] = c + " (Canada)"
##### end
us = [DISPLAY_NAME[c] if c in DISPLAY_NAME  else c for c in us_countries]
us.sort()

all_countries = [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in countries]
all_countries.sort()

##### copy to per_country_rate.py
annotated = [
    "Singapore", "Czechia", "Netherlands", "Italy", "Taiwan", "New Zealand", "Slovakia", "Japan", "South Korea", "Ireland",
    "Denmark", "Serbia", "Vietnam", "Albania", "Spain", "Seychelles", "Luxembourg", "Slovenia", "Greece", "Guam (US)",
    "Sweden", "Russia", "Peru", "UK", "USA", "California", "Virginia", "New York", "Iowa", "Montana", "Nebraska", "Wisconsin",
]
annotated = [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in annotated]
annotated.sort()
#### end of annotated

eu = [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in eu_countries]
eu.sort()

italy_regions = [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in italy_regions]
italy_regions.sort()

# all other sections otheres: all without eu and us
others = list(set(all_countries) - set(eu) - set(us) - set(italy_regions))
others.sort()

col_len = 30

comparison_groups = [
    ("EU", "compare_eu"),
    ("US", "compare_us"),
    ("Italy regions", "italy_regions"),
    ("All countries", "compare_all"),
    ("All (w/o state breakdown)", "compare_all_nobreakdown"),
    ("Annotated only", "compare_annotated"),
    ("Currently increasing cases", "compare_raising"),
    ("Northern hemisphere", "north"),
    ("Southern hemisphere", "south"),
]

def slice_list(l, n):
    r = []
    for i in range(0, len(l), n):
        r.append(l[i:i+n])
    return r

def gen_table(prefix, data, headers, col_len=col_len, table_html_attrs='class="menutable"',
              first_column_html_attrs='class="first"'):
    rows = []
    header_cols = {}

    for i, data_item in enumerate(data):
        header = headers[i]
        header_cols[header] = slice_list(data_item, col_len)

    if table_html_attrs is not None:
        r = f'<table {table_html_attrs}'
    else:
        r = f'<table>'

    # header
    r += "<tr>"

    cols = []
    for header in headers:
        colspan = len(header_cols[header])
        if colspan > 1:
            r += f'<th colspan="{colspan}">{header}</th>'
        else:
            r += f'<th>{header}</th>'
        for col in header_cols[header]:
            cols.append(col)

    rows = []
    for i in range(max(len(col) for col in cols)):
        row = []
        for col in cols:
            try:
                row.append(col[i])
            except IndexError:
                row.append(None)
        rows.append(row)

    # column index of first columns in a section
    # this columns are passed optional first_column_html_attrs
    i = 0
    first_columns = set((i,))
    for header in headers:
        i += len(header_cols[header])
        first_columns.add(i)

    for row in rows:
        r += "<tr>"
        for i, country in enumerate(row):
            html_attr = ""
            if first_column_html_attrs is not None and i in first_columns:
                html_attr = " "+first_column_html_attrs
            if country is None:
                r += f"<td{html_attr}></td>"
            else:
                r += f'<td{html_attr}><a onclick="select_item()" href="{prefix}_{country}.html" target="content">{country}</a></td>'

    r += "</table>"
    return r

new_cases_headers = [
    '<h4>All annotated:</h4>',
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>All</h4>",
]

new_cases = gen_table(
    prefix="new_cases",
    data=[annotated, eu, us, italy_regions, others],
    headers=new_cases_headers,
)

death_toll = gen_table(
    prefix="death_toll",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

reported_deaths = gen_table(
    prefix="reported_deaths",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

reported_cases = gen_table(
    prefix="reported_cases",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

recovered = gen_table(
    prefix="recovered",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

reported_recovered = gen_table(
    prefix="reported_recovered",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

cfr_cases = gen_table(
    prefix="cfr",
    data=[eu, us, italy_regions, others],
    headers=[
    "<h4>EU</h4>",
    "<h4>US</h4>",
    "<h4>Italy regions</h4>",
    "<h4>Other countries:</h4>",
    ],
)

content = {}
content['new_cases'] = new_cases
content['death_toll'] = death_toll
content['recovered'] = recovered
content['reported_cases'] = reported_cases
content['reported_deaths'] = reported_deaths
content['reported_recovered'] = reported_recovered
content['cfr'] = cfr_cases

comparison = {}
for report_type in ["new_cases", "death_toll", "recovered", "reported_cases",
        "reported_deaths", "reported_recovered", "cfr"]:
    div = '<div style="padding-left: 25px">'
    for i, (title, prefix) in enumerate(comparison_groups):
        rate_a = ""
        reported = ""
        if report_type == "new_cases":
            rate_a = f"""<a onclick="select_item()" href="{prefix}_rate.html" target="content">rate</a>,"""
            prefix = prefix+"_cases"
        elif report_type == "death_toll":
            prefix = prefix+"_mortality"
        elif report_type == "recovered":
            prefix = prefix+"_recovered"
        elif report_type == "reported_cases":
            prefix = prefix+"_cases"
            reported = "_reported"
        elif report_type == "reported_deaths":
            prefix = prefix+"_mortality"
            reported = "_reported"
        elif report_type == "reported_recovered":
            prefix = prefix+"_recovered"
            reported = "_reported"
        elif report_type == "cfr":
            prefix = prefix
        p = 0
        if i > 0:
            p = 25
        if report_type == "cfr":
            div += f"""
                <div style="display: inline-block; padding-left: {p}px;">
                <span style="font-size:12px">
                    <b>{title}:</b><br>
                    by date:
                        {rate_a}
                        <a onclick="select_item()" href="{prefix}_cfr.html" target="content">CFR</a>,
                        <a onclick="select_item()" href="{prefix}_cfr_last_21d.html" target="content">21 day CFR</a>,
                        <a onclick="select_item()" href="{prefix}_deaths_population.html" target="content">mortality as % of population</a><br>
                    since 1. case:
                        <a onclick="select_item()" href="{prefix}_cfr_first_day.html" target="content">CFR</a>,
                        <a onclick="select_item()" href="{prefix}_cfr_last_21d_first_day.html" target="content">21 day CFR</a>,
                        <a onclick="select_item()" href="{prefix}_deaths_population_first_day.html" target="content">mortality as % of population</a>
                </span>
                </div>
            """
        else:
            div += f"""
                <div style="display: inline-block; padding-left: {p}px;">
                <span style="font-size:12px">
                    <b>{title}:</b><br>
                    by date:
                        {rate_a}
                        <a onclick="select_item()" href="{prefix}{reported}.html" target="content">daily</a>,
                        <a onclick="select_item()" href="{prefix}{reported}_total.html" target="content">total</a><br>
                    since 1. case:
                        <a onclick="select_item()" href="{prefix}{reported}_first_day.html" target="content">daily</a>,
                        <a onclick="select_item()" href="{prefix}{reported}_total_first_day.html" target="content">total</a>
                </span>
                </div>
        """
    div += "</div>"
    comparison[report_type] = div

index = """
<html>
<head>
<style>
* { font-family: sans-serif; }
#click:hover {
    color: red;
}
#menu {
    position: relative; top: 20px; left: 70px; font-size: 20px; line-height: 30px;
    padding-bottom: 10px;
    margin-bottom: 60px;
    width: auto;
    max-width: 95%;
    background-color: white;
    border: 2px dotted red;
}

a {
    text-decoration: none;
    color: blue;
}
a:visited {
    color: blue;
}
a:hover {
    color: red;
}

.menutable h4 {
    margin-top: 0px;
}

.menutable th {
    padding-left: 25px;
    text-align: left;
}
.menutable td.first {
    padding-left: 25px;
}
.menutable td {
    padding-left: 10px;
}

#topmenutable {
    width: 100%;
    border-collapse: collapse;
}

#topmenutable td {
    width: 50%;
    padding: 5px;
    padding-left: 20px;
}

#topmenutable tr {
    border-top: 2px dotted red;
    border-bottom: 2px dotted red;
}

#topmenutable td:first-child {
    border-right: 2px dotted red;
}

#topmenutable td.wdiv {
    width: 50%;
    padding: 0px;
}

#topmenutable td a {
    color: grey;
}

#topmenutable td div {
    padding: 5px;
    padding-left: 20px;
    cursor: pointer;
}

#topmenutable td div:hover {
    background-color: black;
}

#topmenutable div.topmenu_selected {
    background: rgba(255,0,45,0.09);
}

.hide { display: none; }

</style>
</head>
<body src="padding: 0; margin: 0">

<div id="loading" style="display: none; position: fixed; top: 50%; left: 50%; font-size: 30px; cursor: default; user-select: none;">loading ...</div>
<div id="click" onclick="toggle_menu()" style="position: fixed; top: 20px; left: 20px; font-size: 30px; cursor: default; user-select: none;">&#9776;</div>

<div id="menu" style="display: block">
    <div style="position: relative; font-size: 14px; top: 5px; right: 10px; text-align: right; margin-bottom: 10px;">
        <a href="hcompare.html">hsplit</a> | <a href="vcompare.html">vsplit</a> | <a href="index.html" target="_parent">unsplit</a> | <a href="src/" target="_parent"> src</a>
         | <a href="csv/" target="_parent">csv</a>
    </div>
    <div style="position: relative; width: 100%; font-family: sans-serif">
        <table id="topmenutable">
            <tr style="font-weight: bold">
                <td style="text-align: center">PER CAPITA
                <td style="text-align: center">ABSOLUTE
            <tr>
            <tr>
                <td class="wdiv">
                    <a onclick="topmenu_click('new_cases')">
                    <div class="topmenu_selected topmenu_item" id="topmenu_new_cases">daily cases + R7 <span style="font-size:12px"><i>annotated<b></i></span></div>
                    </a>
                </td>
                <td class="wdiv">
                    <a onclick="topmenu_click('reported_cases')">
                    <div class="topmenu_item" id="topmenu_reported_cases">daily cases</div>
                    </a>
                </td>
            <tr>
                <td class="wdiv">
                    <a onclick="topmenu_click('death_toll')">
                    <div class="topmenu_item" id="topmenu_death_toll">death toll</div>
                    </a>
                </td>
                <td class="wdiv">
                    <a onclick="topmenu_click('reported_deaths')">
                    <div class="topmenu_item" id="topmenu_reported_deaths">death toll</div>
                    </a>
                </td>
            <tr>
                <td class="wdiv">
                    <a onclick="topmenu_click('recovered')">
                    <div class="topmenu_item" id="topmenu_recovered">recovered</div>
                    </a>
                </td>
                <td class="wdiv">
                    <a onclick="topmenu_click('reported_recovered')">
                    <div class="topmenu_item" id="topmenu_reported_recovered">recovered</div>
                    </a>
                </td>
            <tr>
                <td colspan="2" style="text-align: center; border-right: 0px none" class="wdiv">
                    <a onclick="topmenu_click('cfr')">
                    <div class="topmenu_item" id="topmenu_cfr">CFR + death toll as population percentage</div>
                    </a>
                </td>
            </tr>
        </table>
    </div>

"""
for i, prefix in enumerate(content.keys()):
    index += f"""
    <div style="padding: 10px" id="menu_{prefix}" class="menu_countries {'hide' if i > 0 else ''}">
    <div style="padding-left: 125px; margin-bottom: 20px; font-weight: bold;">comparisons:</div>
    {comparison[prefix]}
    <div style="padding-left: 125px; margin-bottom: 20px; margin-top: 20px; font-weight: bold;">country detail:</div>
    {content[prefix]}
    </div>
"""

index += """
</div>

<iframe id="content" src="" name="content" style="width:100%;height:100%; padding: 0; margin: 0; border: 0; display: none"></iframe>

<script>
function toggle_menu() {
    var e = document.getElementById("menu");
    var c = document.getElementById("content");
    if (e.style.display != 'none') {
        e.style.display = 'none';
        c.style.display = 'block';
    } else {
        e.style.display = 'block';
        c.style.display = 'none';
    }
}

function select_item() {
    var ifr=document.getElementById('content');
    ifr.style.display = 'block';
    var e = document.getElementById("menu");
    e.style.display = 'none';
    var l = document.getElementById("loading");
    l.style.display = 'block';
    ifr.onload=function() {
        l.style.display = 'none';
    }
}

function topmenu_click(prefix) {
    var topmenu = document.getElementById("topmenu_"+prefix);
    if (topmenu == null) { return; }
    var content_div = document.getElementById("menu_"+prefix);
    if (content_div == null) { return; }

    var topmenu_items = document.getElementsByClassName("topmenu_item");
    var i;
    for (i=0; i<topmenu_items.length; i++) {
        topmenu_items[i].classList.remove("topmenu_selected");
    }
    topmenu.classList.add("topmenu_selected");

    var menu_items = document.getElementsByClassName("menu_countries");
    for (i=0; i<menu_items.length; i++) {
        menu_items[i].classList.add("hide");
    }

    content_div.classList.remove("hide");
}

</script>

</body>
</html>
"""

with open("per_country/index.html", "w") as f:
    f.write(index)

with open("per_country/hcompare.html", "w") as f:
    f.write("""
<iframe src="index.html" name="top" style="width:100%;height:50%; padding: 0; margin: 0; border: 0;"></iframe>
<iframe src="index.html" name="bottom" style="width:100%;height:50%; padding: 0; margin: 0; border: 0;"></iframe>
""")

with open("per_country/vcompare.html", "w") as f:
    f.write("""
<iframe src="index.html" name="left" style="width:50%;height:100%; padding: 0; margin: 0; border: 0; float:left"></iframe>
<iframe src="index.html" name="right" style="width:50%;height:100%; padding: 0; margin: 0; border: 0; float:left"></iframe>
""")
